library camscan;

import 'package:camscan/camwidget.dart';
import 'package:fast_qr_reader_view/fast_qr_reader_view.dart';
import 'package:flutter/material.dart';

class CamScan {
  Function _globalSuccessCallback = () {};
  Function _globalFailCallback = () {};
  List<Decoder> _decoders;
  String _camWidgetTitle;
  CameraDescription _camera;
  bool _popOnSuccess = true;
  bool _popOnFail = false;
  CamScan(List<Decoder> decoders, {Function successCallback, Function failCallback, String camWidgetTitle = '', bool popOnSuccess = true, bool popOnFail = false})
  {
    _decoders = decoders;
    _globalSuccessCallback = successCallback;
    _globalFailCallback = failCallback;
    _camWidgetTitle = camWidgetTitle;
    _popOnSuccess = popOnSuccess;
    _popOnFail = popOnFail;
    setUpCamera();
  }
  CamScan.fromSingleDecoder(Decoder decoder, {Function successCallback, Function failCallback, String camWidgetTitle = '', bool popOnSuccess = true, bool popOnFail = false})
  {
    _decoders = [decoder];
    _globalSuccessCallback = successCallback;
    _globalFailCallback = failCallback;
    _camWidgetTitle = camWidgetTitle;
    _popOnSuccess = popOnSuccess;
    _popOnFail = popOnFail;
    setUpCamera();
  }
  void setUpCamera() async
  {
    var cams = await availableCameras();
    if (cams.length == 0)
      throw StateError("Can not detect any camera");
    _camera = cams.first;
  }
  void scan(BuildContext currentContext, {bool scanAndPop = true})
  {
    if (_camera == null)
      throw StateError("No available camera! Is the camera setting up?");
    MaterialPageRoute<void> route = MaterialPageRoute<void>(
      builder: (BuildContext context) {
        return CamWidget(_popOnSuccess, _popOnFail, _camWidgetTitle, onResult, _camera);
      },
    );
    Navigator.push(currentContext, route);    
  }
  Future<bool> onResult(String value) async
  {
    for (var decoder in _decoders)
    {
      var result = decoder.tryToDecode(value);
      if (result != null)
      {
        if (!await decoder.tryToInvokeSuccessCB(result))
          await _globalSuccessCallback(result);
        return true;
      }
    }
    await _globalFailCallback();
    return false;
  }
}


class Decoder
{
  Function _successCallback;
  Function _decorder;
  Decoder( Function decorder, {Function successCallback})
  {
    _decorder = decorder;
    _successCallback = successCallback;
  }
  dynamic tryToDecode(String input)
  {
    return _decorder(input);
  }
  Future<bool> tryToInvokeSuccessCB(dynamic result) async
  {
    if (_successCallback == null)
      return false;
    await _successCallback(result);
    return true;
  }
}